#include "sound.h"


#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/datatypes.h>
#include <datatypes/soundclass.h> 

#include <clib/alib_protos.h>
#include <devices/audio.h>


/*--------------------------------------------------------------------------*/

typedef struct
{
	Object* dtp;
	BYTE* address;
	ULONG length;
	UWORD period;
	UWORD volume;
} MyAudio;

MyAudio m_sfx[AMOUNT_OF_SND];

//tablica nazw naszych prbek dwikowych, kolejno
//i ilo musi by taka sama jak w enum SoundNames
char* m_fileNames[AMOUNT_OF_SND] =
{
	"exit.8svx",
	"kill.8svx",
	"knock.8svx",
};

static struct MsgPort* m_pMsgAudioPort = NULL;
static struct IOAudio* m_pAudioIO = NULL;

static UBYTE m_whiChannel[] = {1,2,4,8};

static int initSoundPaula(void);
static void killSoundPaula(void);

/*==========================================================================*/
int InitSound(void)
{
	int iResult = 0;
	int i;

	//wyczyszczenie pola dtp w tablicy sfx
	for (i = 0; i < AMOUNT_OF_SND; ++i)
	{
		m_sfx[i].dtp = 0;
	}

	//zaadowanie plikw dwikowych
	for (i = 0; i < AMOUNT_OF_SND; ++i)
	{
		ULONG period = 0;
		ULONG volume = 0;

		Object* obj = NewDTObject(m_fileNames[i],
			DTA_SourceType, DTST_FILE,
			DTA_GroupID, GID_SOUND,
			TAG_END);

		if (0 == obj)
		{
			return RT_FAILED_SOUND_LOAD;
		}

		m_sfx[i].dtp = obj;
		GetDTAttrs(obj, SDTA_Sample, &m_sfx[i].address, TAG_END);
		GetDTAttrs(obj, SDTA_SampleLength, &m_sfx[i].length, TAG_END);
		GetDTAttrs(obj, SDTA_Period, &period, TAG_END);
		GetDTAttrs(obj, SDTA_Volume, &volume, TAG_END);

		m_sfx[i].volume = (UWORD)volume;
		m_sfx[i].period = (UWORD)period;
	}

	//inicjacja audio device
	initSoundPaula();

	return iResult;
}
/*==========================================================================*/
void KillSound(void)
{
	int i;

	//zamknicie audio.device
	killSoundPaula();

	//zwolnienie zasobw (w tym przypadku prbek dwikowych)
	for (i = 0; i < AMOUNT_OF_SND; ++i)
	{
		DisposeDTObject(m_sfx[i].dtp);
	}

}
/*==========================================================================*/
void PlaySfx(int sfxNumber)
{
	//sprawdzamy czy czasem nie odgrywamy ju czego
	if (0 == CheckIO((struct IORequest*)m_pAudioIO))
	{
		//tak odgrywamy, wic, prosimy audio.device 
		//o zakoczenie odtwarzania sampla
		m_pAudioIO->ioa_Request.io_Command = ADCMD_FINISH;
		m_pAudioIO->ioa_Request.io_Flags  = ADIOF_SYNCCYCLE;
		BeginIO((struct IORequest*)m_pAudioIO);
		WaitIO((struct IORequest*)m_pAudioIO);
	}

	//odtwarzamy sfx o zadanym numerze
	m_pAudioIO->ioa_Request.io_Command = CMD_WRITE;
	m_pAudioIO->ioa_Request.io_Flags = ADIOF_PERVOL;
	m_pAudioIO->ioa_Data = m_sfx[sfxNumber].address;
	m_pAudioIO->ioa_Length = m_sfx[sfxNumber].length;
	m_pAudioIO->ioa_Volume = m_sfx[sfxNumber].volume;
	m_pAudioIO->ioa_Period = m_sfx[sfxNumber].period;
	m_pAudioIO->ioa_Cycles = 1;
	BeginIO((struct IORequest*)m_pAudioIO);
}
/*==========================================================================*/
static int initSoundPaula(void)
{
	int i;
	LONG error;

	//tworzymy msg port
	m_pMsgAudioPort = CreateMsgPort();

	if (0 == m_pMsgAudioPort)
	{
		return -1;
	}

	//umieszczamy go w tworzonym audio requecie
	m_pAudioIO = (struct IOAudio*) 
		CreateIORequest(m_pMsgAudioPort, sizeof(struct IOAudio));

	if (0 == m_pAudioIO)
	{
		return -1;
	}

	//otwieramy audio device i jednoczenie alokujemy kana dwikowy
	m_pAudioIO->ioa_Request.io_Message.mn_ReplyPort = m_pMsgAudioPort;
	m_pAudioIO->ioa_Request.io_Message.mn_Node.ln_Pri = 127;
	m_pAudioIO->ioa_Request.io_Command = ADCMD_ALLOCATE;
	m_pAudioIO->ioa_Request.io_Flags  = ADIOF_NOWAIT;
	m_pAudioIO->ioa_AllocKey = 0;
	m_pAudioIO->ioa_Data = m_whiChannel;
	m_pAudioIO->ioa_Length = sizeof(m_whiChannel);

	error = OpenDevice(AUDIONAME, 0L, (struct IORequest*)m_pAudioIO, 0L);

	if (0 != error)
	{
		return -1;
	}

	return 0;
}
/*==========================================================================*/

static void killSoundPaula(void)
{
	if (m_pAudioIO && m_pAudioIO->ioa_Request.io_Device)
	{
		//zatrzymujemy dzwiki (o ile byy one kiedykolwiek
		//odtwarzane)
		m_pAudioIO->ioa_Request.io_Command = CMD_STOP;
		BeginIO((struct IORequest*)m_pAudioIO);

		//zamykamy device (kana zostanie automatycznie zwolniony)
		CloseDevice((struct IORequest*)m_pAudioIO);
	}

	//niszczymy audio request
	if (m_pAudioIO)
	{

		DeleteIORequest(m_pAudioIO);
	}

	//niszczymy message port dla audio requesta
	if (m_pMsgAudioPort)
	{
		DeleteMsgPort(m_pMsgAudioPort);
	}
}
/*==========================================================================*/
